/*
** Copyright (c) 2005 7B Software Inc.  All rights reserved.
**
** This file is part of XMLP.
**
** XMLP is free software: you can redistribute it and/or modify
** it under the terms of the GNU Lesser General Public License as published by
** the Free Software Foundation, either version 3 of the License, or
** (at your option) any later version.
**
** XMLP is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Lesser General Public License for more details.
**
** You should have received a copy of the GNU Lesser General Public License
** along with XMLP.  If not, see <http://www.gnu.org/licenses/>.
**
** Author contact information:
**   mark.smith@7bsoftware.com
**   http://www.7bsoftware.com
*/

/*
** This file implements a shell for testing XMLP.
*/
#include <tcl.h>
#include <string.h>
#include "xmlptcl.h"
#include <stdlib.h>

#ifdef __WIN32__
# define isatty(X) 0
#else
# include <unistd.h>
#endif

/*
** This is the input loop for the test shell
*/
static char zInputLoop[] =
"if {$argc>0 && [file readable [lindex $argv 0]]} {\n"
   "set argv0 [lindex $argv 0]\n"
   "set argv [lrange $argv 1 end]\n"
   "incr argc -1\n"
   "source $argv0\n"
   "exit\n"
"}\n"
"set tcl_cmd {}\n" 
"while {![eof stdin]} {\n"
  "if {$::tcl_interactive} {\n"
     "if {$tcl_cmd==\"\"} {\n"
       "puts -nonewline {xmlp-test> }\n"
     "} else {\n"
       "puts -nonewline {      ...> }\n"
     "}\n"
     "flush stdout\n"
  "}\n"
  "append tcl_cmd [gets stdin]\\n\n"
  "if {[info complete $tcl_cmd]} {\n"
     "if {[catch {uplevel #0 $tcl_cmd} res]} {\n"
       "puts \"ERROR: $res\"\n"
     "} elseif {$res!=\"\"} {\n"
       "puts $res\n"
     "}\n"
     "set tcl_cmd {}\n"
  "}\n"
"}\n"
"exit\n"
;

/*
** This routine runs first.  
*/
int main(int argc, char **argv){
  Tcl_Interp *interp;
  char *args;
  char buf[100];
  const char *zErrMsg;

  /* Create a Tcl interpreter
  */
  Tcl_FindExecutable(argv[0]);
  interp = Tcl_CreateInterp();
  if( Tcl_PkgRequire(interp, "Tcl", TCL_VERSION, 1)==0 ){
    goto report_error;
  }
  args = Tcl_Merge(argc-1, argv+1);
  Tcl_SetVar(interp, "argv", args, TCL_GLOBAL_ONLY);
  ckfree(args);
  sprintf(buf, "%d", argc-1);
  Tcl_SetVar(interp, "argc", buf, TCL_GLOBAL_ONLY);
  Tcl_SetVar(interp, "argv0", argv[0], TCL_GLOBAL_ONLY);
  Tcl_SetVar(interp, "tcl_interactive", isatty(0)&&isatty(1) ? "1" : "0",
             TCL_GLOBAL_ONLY);

  /* Initialize Tcl and Tk
  */
#ifdef TCL_LIBRARY
  Tcl_SetVar2(interp, "env", "TCL_LIBRARY", TCL_LIBRARY, TCL_GLOBAL_ONLY);
  Tcl_SetVar(interp, "tcl_library", TCL_LIBRARY, TCL_GLOBAL_ONLY);
#endif
  if( Tcl_Init(interp) ) goto report_error;

  /* Start up all extensions.
  */
  Xmlp_Init(interp);

  /* Start getting input from the command line
  */
  if( Tcl_Eval(interp, zInputLoop) ) goto report_error;
  return 0;

report_error:
  zErrMsg = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
  if( zErrMsg==0 ){
    zErrMsg = Tcl_GetStringResult(interp);
  }
  printf("%s", zErrMsg);
  return 1;
}
